
/*
   RC-Sound_Modul
   Ziege-One
   v2.0
 
 Arduino pro Mini 5v/16mHz w/ Atmega 328

Original Git Hub: https://github.com/Ziege-One/RC-Sound_Modul
 
<<<<<<<<<<<<<<<<<<< Abgeänderte Pin Belegung gegeünber dem Original Code >>>>>>>>>>>>>>>>>>>>>>>>>>
 D0: 
 D1: 
 D2: RC in Throttle
 D3: Engine on = pin to VCC
 D4: Sound l on = pin to VCC
 D5: Sound 2 on = pin to VCC
 D6: Sound 3 on = pin to VCC
 D7: Sound 4 on = pin to VCC
 D8: Sound 5 on = pin to VCC
 D9: Sound out 
 D10: CS   SD-Card 
 D11: MOSI SD-Card
 D12: MISO SD-Card
 D13: SCK  (Clk) SD-Card + LED
 
 A0: LED Ausgang für Startkontrolle des Setup Programms
 A1: SetMot => Motor Start Ansteuerung: Auswahl ob der Motor über einen Prop. Kanal oder einen dig. Eingang gesteuert wird. GND = Prop. Kanal (min./max. Ausschlag), VCC = dig. Eingang auf High
 A2: SetSound => Auswahl ob die Zusatzsounds über prop. Kanäle oder dig. Eingänge angesteuert werden. GND = Prop. Kanaläle, VCC = dig. Eingang auf High
 A3: SetEntry => Auswahl ob die Zusatzsounds bei den prop. Kanälen über einzelne Kanäle oder min./max. Auschlag (zwei pro Schalter) aktiviert werden. VCC = pro. Eingang ein Kanal, GND = min./max. Ausschlag ist je Sound.
 A4: Sound 6 on = pin to VCC
 
 */

/********************************************************************************************************************************************
 * 20.07.2021 - V1.7
 * Zweites Debug für den Loop eingebaut
 * Allg. Code Verbesserungen und kleine Bugfix
 *******************************************************************************************************************************************/
 
/********************************************************************************************************************************************
 * 12.07.2021 - V1.6
 * Wartezeit von 10s bis das Setup Programm ausgeführt wird.
 * Werden die Zusatzsound direkt ab  Signal ausgelesen, 
 * müssen die auch zuerst anliegen. Die Wartezeit ist eine Sicherheit, 
 * wenn der Arduino an der Empfängerspannung angeschlossen ist und die Funke noch nicht verbunden.
 *******************************************************************************************************************************************/

/********************************************************************************************************************************************
 * 20.06.2021 - V1.5
 * Bei "Motor Aus" und Abspielen eines Zusatzsaound wurde danach der Motor wieder gestartet. Bug wurde behoben.
 *******************************************************************************************************************************************/

/********************************************************************************************************************************************
 * 20.01.2021 - V1.3
 * SD Kartenlesefehler wird mit dauerblinkenden LED angezeigt
 *******************************************************************************************************************************************/

/********************************************************************************************************************************************
 * 10.12.2020 - V1.1
 * PlaySound Zustände in Array übernommen
 * Integration LED für Sichtbarkeit Abschluss Bootvorgang
 * Dig. Eingänge von "GND" auf "VCC" Aktivierung geändert (passend auf MultiSwitch Print)
 * Erweiterung der Ansteuerung der Sounds über direkte Eingänge ab Empfänger oder dig. Eingänge (Auswahl über Jumper)
 *******************************************************************************************************************************************/


 
// ======== RC-Sound_Modul =======================================

#include <SD.h>                // need to include the SD library
#define SD_ChipSelectPin 10    // using digital pin 10 on arduino nano 328, can use other pins
#include "TMRpcmSpeed.h"       // also need to include this library...
#include <SPI.h>

TMRpcm tmrpcm;   // create an object for use in this sketch

//#define DEBUG          // Serial output setup
//#define DEBUG_Loop     // Serial output loop
//#define Serial_in      // Serial input on (change throttle with keyboard)

unsigned long time = 0;
int numLoop = 0;
int prevThrottle = 0;
int currThrottle = 0;
int playingSound = 0;

bool bPlayArr[9] = {true,false,false, false, false, false, false, false, false};
int shutdowndelay = 15;               // Number of seconds to wait before shutdown sound is played.
boolean shutdowndelay_file = false;   // is there a delay_.txt file 
unsigned int sampleSpeed = 8000;      // Loop samplerate
unsigned int sampleSpeed_max = 32000; // Loop samplerate max
boolean sampleSpeed_max_file = false; // is there a Speed_.txt file

//Pinout
int rc_pin = 2;     // Throttle RC PPM channel
int rc_engine = 3;  // Engine on = pin to VCC
int rc_sound1 = 4;  // Sound l on = pin to VCC
int rc_sound2 = 5;  // Sound 2 on = pin to VCC
int rc_sound3 = 6;  // Sound 3 on = pin to VCC
int rc_sound4 = 7;  // Sound 4 on = pin to VCC
unsigned long duration = 1500;
unsigned long last_duration = 1500;
unsigned long duration_ramp = 50; // max change of duration in one loop
unsigned long timer1;

bool tempMotAus = false;          //Hilfsvariabel für Überprüfung ob das Motorengeräusch aktiv/inaktiv war. Bei Inaktivität und Abspielung von Zusatzssound, wird der Motor nicht mehr gestartet.
                                  //True = Motor war aus



/***********************************
 * Zusätzlich benötigte Variablen
 **********************************/
int rc_sound5 = 8;          // Sound 5 on = pin to VCC
int rc_sound6 = A4;         // Sound 6 on = pin to VCC

bool setSoundLoop = false;           //Damit Zusatzsound im Loop abgespielt wird
bool getMot = true;                  //Variable für Freigabe des Motor Sounds
bool SoundAktivArr[6] = {false, false, false, false, false, false};                             //Array für Freigabe der Sounds nach Abfrage Dig Input/
int SoundEingangArr[6] = {rc_sound1, rc_sound2, rc_sound3, rc_sound4, rc_sound5, rc_sound6};    //Array für physikalische Eingänge
bool SoundEingangArrStat[6] = {false, false, false, false, false, false};                       //Array für Status bei  Eingang Schaltung der Zusatzsounds(Kein Signal = langes Einlesen)
bool SoundMotFreigabe[6] = {false, false, false, false, false, false};                          //Array für Freigabe des Motorgeräusches => sind alle false kommt Motor Sound
volatile bool checkSDCard = true;

int A_setLED = A0;                    //Analog Ausgang für LED Ansteuerung beim Arduino Boot
int A_setMot = A1;                    //Steuern der Signaleingängen: GND => Prop. Kanal; VCC => Dig. Input auf VCC
int A_setSound = A2;                  //Steuern der Signaleingängen: GND => Pro. Kanal; VCC => Dig. Input auf VCC
int A_setEntry = A3;                  //Steuern der Signaleingängen: GND => Einzelne Prop. Kanal (zwei Sounds pro Kanal); VCC => Min./Max. Prop. Kanal Ausschlag




void setup() {

  /**************************************************************
   * Wartezeit von 10s bis das Setup Programm ausgeführt wird.
   **************************************************************/
  long waitTime = millis();
  int WaitingTime = 10000;        //Wartezeit in Millisekunden => Default sind 10sek
  int waitTimeTemp;
  
  while(waitTimeTemp <= WaitingTime)
  {
    waitTimeTemp = millis() - waitTime;  
  }



  pinMode(rc_engine, INPUT_PULLUP);
  pinMode(rc_sound1, INPUT_PULLUP);
  pinMode(rc_sound2, INPUT_PULLUP);
  pinMode(rc_sound3, INPUT_PULLUP);
  pinMode(rc_sound4, INPUT_PULLUP);
  pinMode(rc_sound5, INPUT_PULLUP);
  pinMode(rc_sound6, INPUT_PULLUP);  

  tmrpcm.speakerPin = 9; // Sound out PIN
  Serial.begin(115200);   

    #ifdef DEBUG        // Serial output on 
      Serial.println("<<<<<<<<<<<< Setup Start >>>>>>>>>>>>>>>");
    #endif 

  
  if (!SD.begin(SD_ChipSelectPin)) {  // see if the card is present and can be initialized
    #ifdef DEBUG        // Serial output on     
        Serial.println("SD fail");
    #endif     

    checkSDCard = false;
    //return;   // don't do anything more if not
  }
  else 
  {
    #ifdef DEBUG        // Serial output on     
        Serial.println("SD ok");
    #endif     
  }

  // check for config files
  if(SD.exists("/Delay.txt")) shutdowndelay_file = true;
  if(SD.exists("/Speed.txt")) sampleSpeed_max_file = true;

  if(shutdowndelay_file) 
    shutdowndelay = readIntFromFile("Delay.txt");
  else 
    shutdowndelay = 0; // Default off    
  
  if(sampleSpeed_max_file) 
    sampleSpeed_max = readIntFromFile("Speed.txt");
  else  
    sampleSpeed_max = 32000; // Default max
  
  #ifdef DEBUG        // Serial output on 
    Serial.print("Shutdowndelay : ");
    Serial.println(shutdowndelay);  
    Serial.print("SampleSpeed : ");
    Serial.println(sampleSpeed_max);
  #endif    

  tmrpcm.volume(1);



  
  /*****************************************
  Start Kontrollausgabe nach dem Setup
  *****************************************/
  digitalWrite(rc_engine, LOW);
  digitalWrite(rc_sound1, LOW);
  digitalWrite(rc_sound2, LOW);
  digitalWrite(rc_sound3, LOW);
  digitalWrite(rc_sound4, LOW);
  digitalWrite(rc_sound5, LOW);
  digitalWrite(rc_sound6, LOW);



                #ifdef DEBUG        // Serial output on   
                        Serial.print("Motor Sound Start PWM: ");
                        Serial.println(pulseIn(rc_engine, HIGH));
                #endif
                
  /*****************************************
  Eingänge Initialisieren 
  *****************************************/  
  pinMode(A_setLED, OUTPUT);
  digitalWrite(A_setLED, LOW);
  
  pinMode(A_setMot, INPUT_PULLUP);
  digitalWrite(A_setMot, LOW);   

  pinMode(A_setSound, INPUT_PULLUP);
  digitalWrite(A_setSound, LOW);   

  pinMode(A_setEntry, INPUT_PULLUP);
  digitalWrite(A_setEntry, LOW);


  if(digitalRead(A_setSound) == LOW)        // Inputs Abfragen für Status Array
  {
    
    if(digitalRead(A_setEntry) == LOW)      // //Wenn Min/Max Ausschlag werden nur die ersten J4, J5 und J6 genutzt
    {
             
        for(int i=0; i<6;i++)
        {
          
            if(pulseIn(SoundEingangArr[i], HIGH) > 10)      // Signal liegt an
            {
              SoundEingangArrStat[i] = true;
              i++;
              SoundEingangArrStat[i] = true;

            }
          
        }
      
    }
    else
    {
        
        for(int i=0; i<6;i++)
        {
            if(pulseIn(SoundEingangArr[i], HIGH) > 10)      // Signal liegt an
                SoundEingangArrStat[i] = true;
          
        }
    }


       #ifdef DEBUG        // Serial output on 
       
        Serial.println("Start SoundEingangArrStat Init");
        for(int i=0; i<6;i++)
        {

            if(SoundEingangArrStat[i])
            Serial.print("A");
            else
            Serial.print("I");
            Serial.print(" | ");               
        }
        Serial.println("Ende SoundEingangArrStat Init");

            Serial.println("Start SoundAktivArr Init");
            for(int i=0; i<6;i++)
            {
                if(SoundAktivArr[i])
                Serial.print("A");
                else
                Serial.print("I");
                Serial.print(" | ");
    
                             
            }
            Serial.println("");
            Serial.println("Ende SoundAktivArr Init");
      #endif    
  }

       #ifdef DEBUG        // Serial output on 
        Serial.print(" Motor am Eingang : ");
        Serial.println(pulseIn(rc_pin, HIGH));  

      #endif



    /* 
    *SD Karten Signal Fehler Ausgabe
    */
  if(checkSDCard == true)
  {
   #ifdef DEBUG        // Serial output on 
    Serial.println("SD Card gefunden!");
    #endif

  }
  else
  {

        #ifdef DEBUG        // Serial output on 
        Serial.println("SD Card Fail Ausgabe Blinklicht : ");

       #endif 
     //int gaga = 0;
    //for(gaga=0; gaga<=10000; gaga++)
    while(true)
    {
        digitalWrite(A_setLED, HIGH);
        delay(100);
        digitalWrite(A_setLED, LOW); 
        delay(100);    
    }
    
  }



  /*****************************************
  Boot Sequenz Abschluss
  *****************************************/ 
  
  
      digitalWrite(A_setLED, HIGH);
      delay(1000);
      digitalWrite(A_setLED, LOW);
      delay(1000);
      
      for(int ii = 0; ii < 5; ii++)
      {
        digitalWrite(A_setLED, HIGH);
        delay(200); 
        digitalWrite(A_setLED, LOW);
        delay(200);
      }
    
      digitalWrite(A_setLED, HIGH);
      delay(2000);
      digitalWrite(A_setLED, LOW);






    #ifdef DEBUG        // Serial output on 
      Serial.println("<<<<<<<<<<<< Setup end >>>>>>>>>>>>>>>");
    #endif 

}










void loop() {

     

  if(numLoop == 10000 || SoundMotFreigabe[0] || SoundMotFreigabe[1] || SoundMotFreigabe[2] || SoundMotFreigabe[3] || SoundMotFreigabe[4] || SoundMotFreigabe[5])
  {


        /********************************************************************************************************
         * Auswahl der Zusatzsounds
         * Read A2: Low ist Prop Kanals, High ist Dig. Eingang
         * Read A3: Nur wenn A1 aktiv => Low ist min/max Ausschlag (pro Ausschlag ein Sound), High ist ein Kanal pro Eingang (egal ob max oder min)
         * Einlesen der Eingängen und speichern in Array, Auswertung nur von denen die beim Setup aktiv waren.
         ********************************************************************************************************/

        if(digitalRead(A_setSound) == LOW)    //Prop Kanal
        {
            if(digitalRead(A_setEntry) == LOW)  //Für zwei Sounds wird ein Prop. Kanal genutzt, Schaltung über min oder Max Ausschlag
            {
              //int S_temp = 0;
              int x = 0;
              //for(int x=0; x<3; x++)
              for(int S_temp = 0; S_temp <6; S_temp++)
              {

                if(SoundEingangArrStat[S_temp])      //Status Abfrage ob Signal am Eingang lag beim Setup
                { 
                    
                    noInterrupts();
                    int T_Sound = pulseIn(SoundEingangArr[x], HIGH);    //Zeitkritisches Einlesen des Eingangsignals, ohne noInterrupts() wir ein falscher Werte eingelesen
                    interrupts();
                    
                 
                    if(T_Sound < 1200)
                      SoundAktivArr[S_temp] = true;       //Zusatzsound ist eingeschaltet (Original Code ist auf GND/False gezogen)
                    else
                      SoundAktivArr[S_temp] = false;
   
                              
                    S_temp++;

                    if(T_Sound > 1800)
                      SoundAktivArr[S_temp] = true;       //Zusatzsound ist eingeschaltet (Original Code ist auf GND/False gezogen)
                    else
                      SoundAktivArr[S_temp] = false; 
                  
                  
                }//If SoundEingangArrStat
                else
                {
                   //Erhöhen des Status wenn mal ein Eingang ausgelassen wurde
                  S_temp++;
                }
                
                x++;    //Erhöhung des Counters für Einlesen des PWM Signals
                
              }//Ende For






            }
            else      //Pro Zusatzsound wird ein Prop Kanal genutzt
            {
              for(int xx=0;xx<6;xx++)
              {
                if(SoundEingangArrStat[xx])      //Status Abfrage ob Signal am Eingang lag beim Setup
                {

                    noInterrupts();
                    int T_Sound = pulseIn(SoundEingangArr[xx], HIGH);   //Zeitkritisches Einlesen des Eingangsignals, ohne noInterrupts() wir ein falscher Werte eingelesen
                    interrupts();
                    
                 if(T_Sound <1200 || T_Sound > 1800)
                    SoundAktivArr[xx] = true;       //Zusatzsound ist eingeschaltet (Original Code ist auf GND/False gezogen)
                  else
                    SoundAktivArr[xx] = false;
                }
              }
            }





        #ifdef DEBUG_Loop        // Serial output on  
            Serial.println("Start SoundEingangArrStat Loop");
            for(int i=0; i<6;i++)
            {
                if(SoundEingangArrStat[i])
                Serial.print("A");
                else
                Serial.print("I");
                Serial.print(" | ");
    
                             
            }
            Serial.println("");
            Serial.println("Ende SoundEingangArrStat Loop");


            Serial.println("Start SoundAktivArr Loop");
            for(int i=0; i<6;i++)
            {
                if(SoundAktivArr[i])
                Serial.print("A");
                else
                Serial.print("I");
                Serial.print(" | ");
            
            }
            Serial.println("");
            Serial.println("Ende SoundAktivArr Loop");
        #endif 

        

          
        }
        else  //Dig. Eingänge
        {
            for(int xxx=0;xxx<6;xxx++)
            {
              noInterrupts();
              int T_Sound = digitalRead(SoundEingangArr[xxx]);
              interrupts();
                                  

              if(T_Sound == HIGH)
                SoundAktivArr[xxx] = true;
              else
                SoundAktivArr[xxx] = false;
            }          
        }
    



      /**************************
       * Zusatzsound 1
       **************************/
        
        //if (not digitalRead(rc_sound1))
        if(SoundAktivArr[0])
        {

          if (!bPlayArr[2])
          {

              //if(tmrpcm.isPlaying() || not digitalRead(rc_sound1))
              if(tmrpcm.isPlaying() || SoundAktivArr[0])
              {
                  #ifdef DEBUG_Loop        // Serial output on      
                        Serial.println("sound1.wav");
                  #endif

                //Serial.println("sound1.wav");
                
                 tmrpcm.play("sound1.wav");
                 tmrpcm.speed(tmrpcm.orgsamplerate);

                 tmrpcm.loop(0);
                 while (tmrpcm.isPlaying()) {}
                 

                 //if(not digitalRead(rc_sound2))   //Solang Eingang aktiv ist wird der Sound abgespielt 
                 if(SoundAktivArr[0] == false)
                     tmrpcm.disable();          // disables the timer on output pin and stops the music
                 else
                      SoundMotFreigabe[0] = true;
              }
              else       
                //bPlaySound1 = true;       //Dauerschleife solange Eingang auf GND lieg
                bPlayArr[2] = true;       //bPlaySound2   
          }
        }
        else  
        {
          bPlayArr[2] = false;       //bPlaySound1
          SoundMotFreigabe[0] = false;     //Damit Motor Loop aktiv wird
        }
      



      /**************************
       * Zusatzsound 2
       **************************/
      
        if(SoundAktivArr[1])
        {
          if (!bPlayArr[3])
          {
              if(tmrpcm.isPlaying() || SoundAktivArr[1])
              {
                  #ifdef DEBUG_Loop        // Serial output on      
                        Serial.println("sound2.wav");
                  #endif              

                 tmrpcm.play("sound2.wav");
                 tmrpcm.speed(tmrpcm.orgsamplerate);

                 tmrpcm.loop(0);
                 while (tmrpcm.isPlaying()) {}
                 
                 if(SoundAktivArr[1] == false)  //Solang Eingang aktiv ist wird der Sound abgespielt 
                     tmrpcm.disable();          // disables the timer on output pin and stops the music
                 else
                      SoundMotFreigabe[1] = true;
              }
              else
                bPlayArr[3] = true;       //bPlaySound2  => Dauerschleife solange Eingang auf GND lieg            
          }
        }
        else  
        {
          bPlayArr[3] = false;       //bPlaySound2
          SoundMotFreigabe[1] = false;     //Damit Motor Loop aktiv wird
        }
      
 

       /**************************
       * Zusatzsound 3
       **************************/
      
        if(SoundAktivArr[2])
        {
          if (!bPlayArr[4])
          {

              if(tmrpcm.isPlaying() || SoundAktivArr[2])
              {
                  #ifdef DEBUG_Loop        // Serial output on      
                        Serial.println("sound3.wav");
                  #endif              

                 tmrpcm.play("sound3.wav");
                 tmrpcm.speed(tmrpcm.orgsamplerate);

                 tmrpcm.loop(0);
                 while (tmrpcm.isPlaying()) {}
                 
                 if(SoundAktivArr[2] == false)
                     tmrpcm.disable();          // disables the timer on output pin and stops the music
                 else
                      SoundMotFreigabe[2] = true;
              }
              else
                bPlayArr[4] = true;       //bPlaySound2              
          }
        }
        else  
        {
          bPlayArr[4] = false;       //bPlaySound2
          SoundMotFreigabe[2] = false;     //Damit Motor Loop aktiv wird
        }
      
      



       /**************************
       * Zusatzsound 4
       **************************/
             
        if(SoundAktivArr[3])
        {
          if (!bPlayArr[5])
          {
              if(tmrpcm.isPlaying() || SoundAktivArr[3])
              {
                  #ifdef DEBUG_Loop        // Serial output on      
                        Serial.println("sound4.wav");
                  #endif              

                 tmrpcm.play("sound4.wav");
                 tmrpcm.speed(tmrpcm.orgsamplerate);

                 tmrpcm.loop(0);
                 while (tmrpcm.isPlaying()) {}
                 

                 if(SoundAktivArr[3] == false)
                     tmrpcm.disable();          // disables the timer on output pin and stops the music
                 else
                      SoundMotFreigabe[3] = true;
              }
              else
                bPlayArr[5] = true;       //bPlaySound2              
          }
        }
        else  
        {
          bPlayArr[5] = false;       //bPlaySound2
          SoundMotFreigabe[3] = false;     //Damit Motor Loop aktiv wird
        }
      
 

      /**************************
       * Zusatzsound 5
       **************************/
        
        if(SoundAktivArr[4])
        {
          if (!bPlayArr[6])
          {

              if(tmrpcm.isPlaying() || SoundAktivArr[4])
              {
                  #ifdef DEBUG_Loop        // Serial output on      
                        Serial.println("sound5.wav");
                  #endif              

                 tmrpcm.play("sound5.wav");
                 tmrpcm.speed(tmrpcm.orgsamplerate);

                 tmrpcm.loop(0);
                 while (tmrpcm.isPlaying()) {}
                 
                 if(SoundAktivArr[4] == false)
                     tmrpcm.disable();          // disables the timer on output pin and stops the music
                 else
                     SoundMotFreigabe[4] = true;
              }
              else
                bPlayArr[6] = true;       //bPlaySound2              
          }
        }
        else  
        {
          bPlayArr[6] = false;       //bPlaySound2
          SoundMotFreigabe[4] = false;     //Damit Motor Loop aktiv wird
        }

      /**************************
       * Zusatzsound 6
       **************************/

        if(SoundAktivArr[5])
        {
          if (!bPlayArr[7])
          {
              if(tmrpcm.isPlaying() || SoundAktivArr[5])
              {
                  #ifdef DEBUG_Loop        // Serial output on      
                        Serial.println("sound6.wav");
                  #endif              

                 tmrpcm.play("sound6.wav");
                 tmrpcm.speed(tmrpcm.orgsamplerate);

                 tmrpcm.loop(0);
                 while (tmrpcm.isPlaying()) {}
                 

                 if(SoundAktivArr[5] == false)
                     tmrpcm.disable();          // disables the timer on output pin and stops the music
                 else
                     SoundMotFreigabe[5] = true;
              }
              else
                bPlayArr[7] = true;       //bPlaySound2              
          }
        }
        else  
        {
          bPlayArr[7] = false;       //bPlaySound2
          SoundMotFreigabe[5] = false;     //Damit Motor Loop aktiv wird
        }






/******************************************
 * Motoren Geräusch
 ******************************************/

      /*****************************************************
      * Abfrage ob Motor Start über Dig. Inputs oder 
      ******************************************************/
      if(digitalRead(A_setMot) == LOW)              
      {
        //Auswahl über Prop. Eingang
           noInterrupts();
           int T_Mot = pulseIn(rc_engine, HIGH);
           interrupts();

                #ifdef DEBUG_Loop        // Serial output on   
                        Serial.print("Motor Sound PWM: ");
                        Serial.println(T_Mot);
                #endif
                                    
          if(T_Mot < 1200 || T_Mot > 1800)
            getMot = true;
          else
            getMot = false;        
      }
      else
      {
           noInterrupts();
           int T_Mot = digitalRead(rc_engine);
           interrupts();

                #ifdef DEBUG_Loop        // Serial output on   
                        Serial.println("Motor Sound Digital");
                #endif
                                  
        //Auswahl über Dig. Eingang        
        if(T_Mot == HIGH)
          getMot = true;
        else 
          getMot = false;
      }







      if(getMot == true && not SoundMotFreigabe[0] && not SoundMotFreigabe[1] && not SoundMotFreigabe[2] && not SoundMotFreigabe[3] && not SoundMotFreigabe[4] && not SoundMotFreigabe[5])
      {
                
                #ifdef DEBUG_Loop        // Serial output on   
                        Serial.println("Motor Sound an");
                #endif
 
        /*
        * Überprüfung ob Motor vor dem Zusatzsound aktiv war => Motor war nicht aktiv
        */
        //Erster Start
        if(!bPlayArr[8] && tempMotAus == false)
        {
          bPlayArr[8] = true;       //bPlayEngine
          bPlayArr[0] = false;    //bPlayStart   
        }  

     
      }
      else
      {
        bPlayArr[8] = false;       //bPlayEngine
        currThrottle = 0;
        bPlayArr[1] = false;       //bPlayNormal
      }



  }//Ende NumLoop


  
  ++numLoop;



  
  if (numLoop == 30000 && not SoundMotFreigabe[0] && not SoundMotFreigabe[1] && not SoundMotFreigabe[2] && not SoundMotFreigabe[3] && not SoundMotFreigabe[4] && not SoundMotFreigabe[5]) 
  {

      /**********************************************
       * Debug Geschwindigkeits Änderung mit Keyboard
       **********************************************/
       
      #ifndef Serial_in         // Serial input off (change throttle with keyboard)   
          noInterrupts();
            duration = pulseIn(rc_pin, HIGH);      //Read throttel value
          interrupts();
          
          if (duration > last_duration)          //throttel go up
          {
              if (last_duration + duration_ramp < duration)
                duration = last_duration + duration_ramp;
              else
                duration = duration;
          }
          else                                   //throttel go down
          {
              if (last_duration - duration_ramp > duration)
                duration = last_duration - duration_ramp;
              else
                duration = duration;
          }
          
          last_duration = duration;
      #endif    

      

      /**********************************************
       * Ende Debug Geschwindigkeits Änderung mit Keyboard
       **********************************************/


      if(getMot == false)
          duration = 1500; 


      /* */
      #ifdef Serial_in        // Serial input on (change throttle with keyboard)
        if(Serial.available()) 
        {                          //Send commands over serial to play
              switch(Serial.read()){;  
                case '0': duration = 950; break; 
                case '1': duration = 1050; break;
                case '2': duration = 1150; break;
                case '3': duration = 1250; break;
                case '4': duration = 1450; break;
                case '5': duration = 1500; break;
                case '6': duration = 1550; break;
                case '7': duration = 1750; break;
                case '8': duration = 1850; break;        
                case '9': duration = 1950; break;         
                case ',': duration = 2050; break;   
              }
        }
      #endif

     
      
          numLoop = 0;

      #ifdef DEBUG_Loop          // Serial output on  

        
          Serial.print("Pulse in : ");
          Serial.print(duration);
          Serial.print(" | SampleSpeed : ");
          Serial.print(sampleSpeed);
          Serial.print(" | currThrottle : ");
          Serial.print(currThrottle);
          Serial.print(" | prevThrottle : ");
          Serial.print(prevThrottle);          
          Serial.println(" | Playing : ");
          
           
      #endif 


          
          /**********************************
           * Erneutes Starten des Motors wenn dieser mal automatisch nach x-sekunden ausgeschaltet wurde
           **********************************/
          if( tempMotAus && (duration <= 1480 || duration > 1520) && currThrottle == 0 && prevThrottle == 0 )
          {

            bPlayArr[0] = true;
            tempMotAus =  false;
            
          }

          
      
          // Code to handle statup and //

          if ( ! bPlayArr[0] && tempMotAus == false)
          {
            currThrottle = 1; // Play statup sound
            bPlayArr[0] = true;    //bPlayStart
          }
          // Shutdowndelay 0 = off
          if ( (millis() - timer1) > (shutdowndelay * 1000) && currThrottle == 2 && (shutdowndelay > 0)) 
          {
            currThrottle = 0;
            bPlayArr[1] = false;       //bPlayNormal

          }
      
          /* Reverse throttle */
          if (duration <= 1480) 
          {
            if(bPlayArr[1])
            {
              currThrottle = 3;
              sampleSpeed = map(duration,1480,1000,tmrpcm.orgsamplerate,sampleSpeed_max);
              tmrpcm.speed(sampleSpeed);
            }
            else
             bPlayArr[0] = false;    //bPlayStart

          };
      
          /* Throttle stick center - Play sound orignal samplerate  */
          if ((duration > 1480) && (duration <= 1520) && bPlayArr[1]) 
          {
            currThrottle = 2;
            tmrpcm.speed(tmrpcm.orgsamplerate);
          };


      
          /* Forard throttle */
          if (duration > 1520) 
          {
            if(bPlayArr[1])
            {
              currThrottle = 3;
              sampleSpeed = map(duration,1520,2000,tmrpcm.orgsamplerate,sampleSpeed_max);
              tmrpcm.speed(sampleSpeed);
            }
            else
              bPlayArr[0] = false;    //bPlayStart
          };




        if ((currThrottle != prevThrottle) or (tmrpcm.isPlaying() == 0)) 
        {
          timer1 = millis();
          tmrpcm.loop(1);



      /**********************************************
       * Motor Geräusch wird ausgeschaltet
       **********************************************/
        
              if ((currThrottle == 0) && (prevThrottle > 0)) 
              {
                tmrpcm.play("stop.wav");
                tmrpcm.speed(tmrpcm.orgsamplerate);
                
                #ifdef DEBUG_Loop        // Serial output on        
                        Serial.println("stop.wav");
                #endif 
        
                tmrpcm.loop(0);
                while (tmrpcm.isPlaying()) {}
                tmrpcm.disable();  // disables the timer on output pin and stops the music
                prevThrottle = currThrottle;
                bPlayArr[1] = false;       //bPlayNormal

                tempMotAus = true;

              }







      /**********************************************
       * Motor Geräusch wird gestartet
       **********************************************/              
                if (currThrottle == 1) {
                  if (currThrottle > prevThrottle) 
                  {
                    tmrpcm.play("start.wav");
                    tmrpcm.speed(tmrpcm.orgsamplerate);
                    
                    #ifdef DEBUG_Loop        // Serial output on          
                              Serial.println("start.wav");
                    #endif           
                    
                    tmrpcm.loop(0);
                    while (tmrpcm.isPlaying()) {}
                  }

                  bPlayArr[1] = true;       //bPlayNormal
                  playingSound = 1;
                  prevThrottle = currThrottle;
                }











    
      /**********************************************
       * Motor Geräusch loop
       **********************************************/     
                      
                if (((currThrottle == 2) && ((prevThrottle == 1) || (tmrpcm.isPlaying() == 0))) || ((currThrottle == 3) && ((prevThrottle == 1) || (tmrpcm.isPlaying() == 0)))) 
                {
    
                  tmrpcm.play("loop.wav");
                  
                    #ifdef DEBUG_Loop        // Serial output on        
                            Serial.println("loop.wav");
                    #endif      
                    
                  prevThrottle = currThrottle;
                  playingSound = 1;
                }
                if (currThrottle == 2) 
                { 
                  prevThrottle = currThrottle;
                  playingSound = 1;
                }
                if (currThrottle == 3) 
                { 
                  prevThrottle = currThrottle;
                  playingSound = 2;
                }
        }//Ende neuer Throttle Wert


        
      #ifdef DEBUG_Loop        // Serial output on
          Serial.print("Motors Sound is Playing => ");      
          Serial.println(tmrpcm.isPlaying());
      #endif       

      
  }     //End NumLoop = 3000
} //Ende Voide Loop






// read Int from textfile via SD-card
int readIntFromFile(const char filename[])
{
  uint8_t pos=0;
  uint8_t BUFSIZE=20;
  char buffer[BUFSIZE]; 
  
  File dataFile = SD.open (filename, FILE_READ);
  if(!dataFile) return 0;
  while ((buffer[pos] = dataFile.read()) >= 0 && pos < (BUFSIZE-1)) pos++;
  dataFile.close();
  return (atoi(buffer));
}
